#include <stdlib.h>
#include <malloc.h>
#include <windows.h>
#include <stdio.h>
#include "ceconf.h"


#if defined(UNDER_CE)
#pragma warning(disable:4761)
#endif

struct cefile {
	HANDLE handle;
};


/* Private function definitions */
static WCHAR *StrToUni( const char * str, WCHAR * uni, int buflen );

static HANDLE openfiles[ MAX_OPENFILES ];

int open( const char * filename, int oflag, int pmode )
{
	int i;
	DWORD access = 0, create =0;
	WCHAR fname[ _MAX_PATH ];

	(void)pmode;

	if( oflag & O_CREAT )
		create = CREATE_ALWAYS;

	if( oflag & O_WRONLY || oflag & O_RDWR )
		access |= GENERIC_WRITE;

	if( oflag & O_RDONLY || oflag & O_RDWR )
		access |= GENERIC_READ;

	for( i = 0; i < MAX_OPENFILES; i++ )
		if( !openfiles[ i ] )
			break;

	if( i == MAX_OPENFILES )
		return -1;

	MultiByteToWideChar( CP_ACP, 0, filename, -1, fname, _MAX_PATH );

	if( ( openfiles[ i ] = CreateFile( fname, access, 0, NULL, create, 0, NULL ) ) == INVALID_HANDLE_VALUE )
		return -1;

	return i;
}

int close( int handle )
{
	CloseHandle( openfiles[ handle ] );
	openfiles[ handle ] = NULL;
	return 0;
}

int read( int handle, void *buffer, unsigned int count )
{
	BOOL success;
	DWORD readd;
	success = ReadFile(openfiles[handle], buffer, count, &readd, NULL);
	if(!success || !readd) return 0;
	else return readd;
}

size_t write( int handle, const void *buffer, size_t count )
{
	BOOL success;
	DWORD written;
	success = WriteFile(openfiles[handle], buffer, count, &written, NULL);
	if(success == FALSE) return 0;
	else return written;
}

long lseek( int handle, long offset, int origin )
{
	struct cefile ce = { openfiles[handle] };
	return fseek((FILE *)&ce, offset, origin);
}


int unlink( const char * pathname )
{
	WCHAR * w_filename;
	int length;
	int retval;

	length = strlen( pathname );
	w_filename = (WCHAR *)malloc( ( length + 1 ) * sizeof( WCHAR ) );

	MultiByteToWideChar( CP_ACP, 0, pathname, -1, w_filename, length );

	retval = DeleteFile( w_filename );

	free( w_filename );

	return retval;
}

int rename( const char * oldname, const char * newname )
{
	WCHAR * w_old, * w_new;
	int oldlen, newlen;
	int retval;

	oldlen = strlen( oldname );
	newlen = strlen( newname );

	w_old = (WCHAR *)malloc( ( oldlen + 1 ) * sizeof( WCHAR ) );
	w_new = (WCHAR *)malloc( ( newlen + 1 ) * sizeof( WCHAR ) );

	MultiByteToWideChar( CP_ACP, 0, oldname, -1, w_old, oldlen );
	MultiByteToWideChar( CP_ACP, 0, newname, -1, w_new, newlen );

	retval = MoveFile( w_old, w_new );

	free( w_old );
	free( w_new );

	return retval;
}


char * strrchr( const char * string, int c )
{
	WCHAR * unistr;
	WCHAR * w;
	int length;

	length = strlen( string );

	unistr = (WCHAR *)malloc( ( length + 1 ) * sizeof( WCHAR ) );

	w = wcsrchr( StrToUni( string, unistr, length ), c );

	if( w ) {
		length = (((unsigned long)w - (unsigned long)unistr) / 2);
		free( unistr );
		return (char *)(string + length);
	}

	free( unistr );

	return NULL;
}

WCHAR * StrToUni( const char * str, WCHAR * uni, int buflen )
{
	MultiByteToWideChar( CP_ACP, 0, str, -1, uni, buflen );
	return uni;
}

void regularize( char * s )	/* normalize file name */
{
	register char *lp;

	for (lp = s; *lp; lp++)
	    if ( *lp == '?' || *lp == '"' || *lp == '\\' ||
		 *lp == '/' || *lp == '>' || *lp == '<'  ||
		 *lp == '*' || *lp == '|' || *lp == ':'  || (*lp > 127))
			*lp = '_';
	
}

int isspace( int c )
{
	char str[ 2 ] = { c, '\0' };
	WCHAR unistr[ 2 ];

	StrToUni( str, unistr, 2 );
	return iswspace( unistr[ 0 ] );
}



struct tm gtm;
SYSTEMTIME st;

time_t time( time_t *timer )
{
	GetLocalTime(&st);
	return (st.wYear-1970)* 31795200 + (st.wMonth-1) * 30 * 24* 60 * 60 + (st.wDay-1) * 24 * 60 * 60 + st.wHour * 60 * 60 + st.wMinute * 60 + st.wSecond;
}

struct tm *localtime( const time_t *timer )
{
	gtm.tm_sec = st.wSecond;
	gtm.tm_min = st.wMinute;
	gtm.tm_hour = st.wHour;
	gtm.tm_mday = st.wDay;
	gtm.tm_mon = st.wMonth;
	gtm.tm_year = st.wYear - 1900;
	gtm.tm_wday = st.wDayOfWeek;
	//tm_yday Day of year (0  365; January 1 = 0)
	return &gtm;
}

void abort(void)
{
	exit( 0 );
}

void nethack_exit(int code)
{
	ExitThread( code );
}

int isupper( int c )
{
	char str[2];
	WCHAR unistr[2];
	str[0] = c;
	str[1] = 0;

	StrToUni(str, unistr, 2);
	return iswupper(unistr[0]);
}

void error(const char *str, ...)
{
	// todo: printf in window
}

char *getenv( const char *varname )
{
	// Always return "not found" on Windows CE
	return NULL;
}

void more()
{
}

void rewind( FILE *stream )
{
}
